#!/usr/bin/env lua
local config = require("lime.config")
local utils = require("lime.utils")
local iwinfo = require("iwinfo")
local uci = config.get_uci_cursor()

mesh_blocker = {}

function get_mesh_plink(ifname, mac)
  fd = io.popen(
    "iw dev " ..ifname .. " station get " .. mac .. " | " ..
    "grep \"mesh plink:\" | awk '{print $3}'"
  )
  local plink = fd:read('*l')
  fd:close()
  return plink
end

function block_mesh_peer(ifname, mac)
  local exit_code = os.execute(
    "iw dev " ..ifname.. " station set " .. mac .. " plink_action block"
  )
  if exit_code == 0 then
    utils.log("Station " .. mac .. " at " .. ifname .. " has been BLOCKED")
  else
    utils.log("Error blocking station " .. mac .. " at " .. ifname)
  end
end

function open_mesh_peer(ifname, mac)
  local exit_code = os.execute(
    "iw dev " ..ifname.. " station set " .. mac .. " plink_action open"
  )
  if exit_code == 0 then
    utils.log("Station " .. mac .. " at " .. ifname .. " has been OPENED")
  else
    utils.log("Error opening station " .. mac .. " at " .. ifname)
  end
end

function mesh_blocker.blacklist()
    local mesh_blockers_configs = uci:get_all('mesh-blocker')
    for _, config in pairs(mesh_blockers_configs) do
      local ifname = config.ifname
      local blacklist = {}
      for _, mac in pairs(config.macs) do
        table.insert(blacklist, mac:lower())
      end
      local assoclist = iwinfo.type(ifname) and 
        iwinfo[iwinfo.type(ifname)].assoclist(ifname) or {}
      for mac, _ in pairs(assoclist) do
        local plink_status = get_mesh_plink(ifname, mac)
        local black_listed = utils.has_value(blacklist, mac:lower())
          or utils.has_value(blacklist, mac:lower())
        if black_listed and plink_status ~= 'BLOCKED' then
          block_mesh_peer(ifname, mac)
        elseif not black_listed and plink_status == 'BLOCKED' then
          open_mesh_peer(ifname, mac)
        end
      end
    end    
end


return mesh_blocker
